@@ -1,11 +1,17 @@
 from functools import wraps
+from inspect import signature
 from typing import Awaitable, List as ListT
-from fastapi import FastAPI, Request, Response
+from fastapi import FastAPI, Request
 from pydantic import BaseModel
 
 from .http import MetroHTTP
 from .models import Bot, ListUpdate
 
+try:
+    from fastapi.responses import ORJSONResponse as JSONResp
+except:
+    from fastapi.responses import JSONResponse as JSONResp
+
 DEFAULT_TAGS = ["Metro (Integrase)"]
 """Default tags for the integrase routes"""
 
@@ -51,6 +57,7 @@ class Metro():
         self.domain = domain
         self._urls = {}
         self._app = app
+        self.wrapped = {}
     
     async def paginate(self, func: Awaitable, *, limit: int = 50):
         """
@@ -102,11 +109,16 @@ class Metro():
         if not self._app:
             raise ValueError("App must be passed in order to use this method")
 
+        # Check the args of func
+        func_sig = signature(func)
+
+        if not func_sig.parameters.get("request"):
+            raise ValueError("Function must take request")
+
         @wraps(func)
-        async def metro_f(request: Request, response: Response, bot: Bot, *args, **kwargs):
+        async def metro_f(request: Request, bot: Bot, *args, **kwargs):
             if request.headers.get("Authorization") != self.http.secret_key:
-                response.status_code = 401
-                return {"detail": "Invalid secret key"}
+                return JSONResp({"detail": "Invalid secret key"}, status_code=401)
 
             return await func(bot, *args, **kwargs)
 
@@ -114,6 +126,8 @@ class Metro():
 
         self._urls[name] = f"{self.domain}{url}"
 
+        self.wrapped[name] = metro_f
+
     def claim(self, *, tags: ListT[str] = DEFAULT_TAGS, url: str = "/metro/claim"):
         """Claim API Decorator"""
 
